@@ -13,7 +13,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License 
+   You should have received a copy of the GNU General Public License
    along with this library; see the file COPYING. If not, write to the
    Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA.
@@ -79,8 +79,7 @@ union float_long
   };
 
 /* compare two floats */
-_Bool
-__fseq (float a1, float a2)
+_Bool __fseq (float a1, float a2) __SDCC_FLOAT_NONBANKED
 {
   volatile union float_long fl1, fl2;
 
