@@ -113,7 +113,7 @@ machine(struct mne *mp)
         case S_ACC:
                 t = addr(&e);
                 if (t != S_A)
-                        aerr();
+                        xerr('a', "Argument must be A.");
                 outab(op);
                 break;
 
@@ -144,13 +144,13 @@ machine(struct mne *mp)
                 case S_DPTR:
                         if (op != 0)
                                 /* only INC (op=0) has DPTR mode */
-                                aerr();
+                                xerr('a', "DPTR allowed only in INC instruction.");
                         else
                                 outab( 0xA3);
                         break;
 
                 default:
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 }
                 break;
 
@@ -158,7 +158,7 @@ machine(struct mne *mp)
                 /* A,#imm; A,direct; A,@R0; A,@R1; A,R0 to A,R7 */
                 t = addr(&e);
                 if (t != S_A)
-                        aerr();
+                        xerr('a', "First argument must be A.");
                 comma(1);
                 t1 = addr(&e1);
 
@@ -183,7 +183,7 @@ machine(struct mne *mp)
                         break;
 
                 default:
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 }
                 break;
 
@@ -212,7 +212,7 @@ machine(struct mne *mp)
                                 break;
 
                         default:
-                                aerr();
+                                xerr('a', "Invalid Addressing Mode.");
                         }
                         break;
 
@@ -238,14 +238,14 @@ machine(struct mne *mp)
                                 break;
 
                         default:
-                                aerr();
+                                xerr('a', "Invalid Addressing Mode.");
                         }
                         break;
 
                 case S_C:
                         /* XRL has no boolean version.  Trap it */
                         if (op == 0x60)
-                                aerr();
+                                xerr('a', "XRL does not support boolean.");
 
                         switch (t1) {
                         case S_DIR:
@@ -260,12 +260,12 @@ machine(struct mne *mp)
                                 break;
 
                         default:
-                                aerr();
+                                xerr('a', "Invalid Addressing Mode.");
                         }
                         break;
 
                 default:
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 }
                 break;
 
@@ -273,7 +273,7 @@ machine(struct mne *mp)
                 /* A,direct; A,@R0; A,@R1; A,R0 to A,R7 */
                 t = addr(&e);
                 if (t != S_A)
-                        aerr();
+                        xerr('a', "First argument must be A.");
                 comma(1);
                 t1 = addr(&e1);
 
@@ -293,7 +293,7 @@ machine(struct mne *mp)
                         break;
 
                 default:
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 }
                 break;
 
@@ -326,7 +326,7 @@ machine(struct mne *mp)
                                 break;
 
                         default:
-                                aerr();
+                                xerr('a', "Invalid Addressing Mode.");
                         }
                         break;
 
@@ -348,7 +348,7 @@ machine(struct mne *mp)
                                 break;
 
                         default:
-                                aerr();
+                                xerr('a', "Invalid Addressing Mode.");
                         }
                         break;
 
@@ -389,7 +389,7 @@ machine(struct mne *mp)
                                 break;
 
                         default:
-                                aerr();
+                                xerr('a', "Invalid Addressing Mode.");
                         }
                         break;
 
@@ -411,26 +411,26 @@ machine(struct mne *mp)
                                 break;
 
                         default:
-                                aerr();
+                                xerr('a', "Invalid Addressing Mode.");
                         }
                         break;
 
                 case S_C:
                         if ((t1 != S_DIR) && (t1 != S_EXT))
-                                aerr();
+                                xerr('a', "Second argument must be an address.");
                         outab(0xA2);
                         outrb(&e1, R_PAG0);
                         break;
 
                 case S_DPTR:
                         if (t1 != S_IMMED)
-                                aerr();
+                                xerr('a', "#__ is required second argument.");
                         outab(0x90);
                         outrw(&e1, R_NORM);
                         break;
 
                 default:
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 }
                 break;
 
@@ -438,7 +438,7 @@ machine(struct mne *mp)
                 /* Branch on bit set/clear */
                 t = addr(&e);
                 if ((t != S_DIR) && (t != S_EXT))
-                        aerr();
+                        xerr('a', "Argument must be an address.");
                 /* sdcc svn rev #4994: fixed bug 1865114 */
                 comma(1);
                 expr(&e1, 0);
@@ -450,7 +450,7 @@ machine(struct mne *mp)
                         v1 = (int) (e1.e_addr - dot.s_addr - 1);
                         /* sdcc svn rev #602: Fix some path problems */
                         if (pass == 2 && ((v1 < -128) || (v1 > 127)))
-                                aerr();
+                                xerr('a', "Branching Range Exceeded.");
                         outab(v1);
                 } else {
                         outrb(&e1, R_PCR);
@@ -469,7 +469,7 @@ machine(struct mne *mp)
                         v1 = (int) (e1.e_addr - dot.s_addr - 1);
                         /* sdcc svn rev #602: Fix some path problems */
                         if (pass == 2 && ((v1 < -128) || (v1 > 127)))
-                                aerr();
+                                xerr('a', "Branching Range Exceeded.");
                         outab(v1);
                 } else {
                         outrb(&e1, R_PCR);
@@ -499,25 +499,25 @@ machine(struct mne *mp)
                                 outrb(&e1, R_PAG0);
                         }
                         else
-                                aerr();
+                                xerr('a', "Invalid Addressing Mode.");
                         break;
 
                 case S_AT_R:
                         outab(op + 6 + e.e_addr);
                         if (t1 != S_IMMED)
-                                aerr();
+                                xerr('a', "#__ is required second argument.");
                         outrb(&e1, R_NORM);
                         break;
 
                 case S_REG:
                         outab(op + 8 + e.e_addr);
                         if (t1 != S_IMMED)
-                                aerr();
+                                xerr('a', "#__ is required second argument.");
                         outrb(&e1, R_NORM);
                         break;
 
                 default:
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                         break;
                 }
 
@@ -526,7 +526,7 @@ machine(struct mne *mp)
                         v1 = (int) (e2.e_addr - dot.s_addr - 1);
                         /* sdcc svn rev #602: Fix some path problems */
                         if (pass == 2 && ((v1 < -128) || (v1 > 127)))
-                                aerr();
+                                xerr('a', "Branching Range Exceeded.");
                         outab(v1);
                 } else {
                         outrb(&e2, R_PCR);
@@ -554,7 +554,7 @@ machine(struct mne *mp)
                         break;
 
                 default:
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 }
 
                 /* branch destination */
@@ -563,7 +563,7 @@ machine(struct mne *mp)
                         v1 = (int) (e1.e_addr - dot.s_addr - 1);
                         /* sdcc svn rev #602: Fix some path problems */
                         if (pass == 2 && ((v1 < -128) || (v1 > 127)))
-                                aerr();
+                                xerr('a', "Branching Range Exceeded.");
                         outab(v1);
                 } else {
                         outrb(&e1, R_PCR);
@@ -576,7 +576,7 @@ machine(struct mne *mp)
                 /* @A+DPTR */
                 t = addr(&e);
                 if (t != S_AT_ADP)
-                        aerr();
+                        xerr('a', "JMP @A+DPTR is the only allowed mode.");
                 outab(op);
                 break;
 
@@ -584,7 +584,7 @@ machine(struct mne *mp)
                 /* A,@A+DPTR  A,@A+PC */
                 t = addr(&e);
                 if (t != S_A)
-                        aerr();
+                        xerr('a', "First argument must be A.");
                 comma(1);
                 t1 = addr(&e1);
                 if (t1 == S_AT_ADP)
@@ -592,7 +592,7 @@ machine(struct mne *mp)
                 else if (t1 == S_AT_APC)
                         outab(0x83);
                 else
-                        aerr();
+                        xerr('a', "MOVC A,@A+DPTR; A,@A+PC are the allowed modes.");
                 break;
 
         case S_MOVX:
@@ -613,7 +613,7 @@ machine(struct mne *mp)
                                 break;
 
                         default:
-                                aerr();
+                                xerr('a', "Second argument must be @DPTR or @Rn.");
                         }
                         break;
 
@@ -621,18 +621,18 @@ machine(struct mne *mp)
                         if (t1 == S_A)
                                 outab(0xF0);
                         else
-                                aerr();
+                                xerr('a', "Second argument must A.");
                         break;
 
                 case S_AT_R:
                         if (t1 == S_A)
                                 outab(0xF2 + e.e_addr);
                         else
-                                aerr();
+                                xerr('a', "Second argument must A.");
                         break;
 
                 default:
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 }
                 break;
 
@@ -640,7 +640,7 @@ machine(struct mne *mp)
         case S_AB:
                 t = addr(&e);
                 if (t != S_RAB)
-                        aerr();
+                        xerr('a', "A,B is the only valid argument.");
                 outab(op);
                 break;
 
@@ -666,7 +666,7 @@ machine(struct mne *mp)
                         break;
 
                 default:
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 }
                 break;
 
@@ -685,7 +685,7 @@ machine(struct mne *mp)
                         break;
 
                 default:
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 }
                 break;
 
@@ -697,7 +697,7 @@ machine(struct mne *mp)
                         e.e_mode = S_DIR;
                 } else
                 if ((t != S_DIR) && (t != S_EXT)) {
-                        aerr();
+                        xerr('a', "Argument must be an address.");
                         break;
                 }
                 outab(op);
@@ -708,7 +708,7 @@ machine(struct mne *mp)
         case S_XCHD:
                 t = addr(&e);
                 if (t != S_A)
-                        aerr();
+                        xerr('a', "First argument must A.");
                 comma(1);
                 t1 = addr(&e1);
                 switch (t1) {
@@ -717,13 +717,13 @@ machine(struct mne *mp)
                         break;
 
                 default:
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 }
                 break;
 
         default:
                 opcycles = OPCY_ERR;
-                err('o');
+                xerr('o', "Internal Opcode Error.");
                 break;
         }
         if (opcycles == OPCY_NONE) {
