@@ -209,7 +209,7 @@ enum {
   W_POSSBUG2                    = 179, /* possible bug, new format */
   W_COMPLEMENT                  = 180, /* ~bit can give unexpected results */
   E_SHADOWREGS_NO_ISR           = 181, /* shadowregs keyword following non-ISR function */
-  W_SFR_ABSRANGE                = 182, /* sfr at address out of range */
+  W_SFR_ABSRANGE                = 182, /* __sfr at address out of range */
   E_BANKED_WITH_CALLEESAVES     = 183, /* banked and callee-saves mixed */
   W_INVALID_INT_CONST           = 184, /* invalid integer literal string */
   W_CMP_SU_CHAR                 = 185, /* comparison of 'signed char' with 'unsigned char' requires promotion to int */
@@ -324,6 +324,16 @@ enum {
   W__SDCC_EXTERNAL_STARTUP_DEF  = 294, /* _sdcc_external_startup function definition - probable deprecated old-style variant of __sdcc_external_startup */
   E_NO_SKIP_TARGET              = 295, /* target label missing for skip instruction */
   W_SDCCCALL_STD_LIB_CRT0       = 296, /* non-default sdcccall specified, but default stdlib or crt0 */
+  W_PEEPHOLE_RULE_LIMIT         = 297, /* peephole rule application limit reached */
+  W_DATA_ABSRANGE               = 298, /* __data at address out of range */
+  W_IDATA_ABSRANGE              = 299, /* __idata at address out of range */
+  W_CASE_RANGE_EMPTY            = 300, /* empty case range specified */
+  E_CASE_RANGE_C2Y              = 301, /* case range expressions require C2y or later */
+  E_GENERIC_WITH_TYPENAME_C2Y   = 302, /* generic selection based on a type name requires C2y or later */
+  E_MIXED_FUNCTION_STYLES       = 303, /* function mixes ISO and K&R style */
+  E_ENUM_TYPE_SPECIFIER_C23     = 304, /* enum type specifiers require C23 or later */
+  E_ENUM_UNDERLYING_TYPE        = 305, /* enum's underlying type must be an integer type and cannot be bit-precise or an enum */
+  E_ENUM_TYPE_RANGE_TOO_SMALL   = 306, /* the enum's underlying type cannot represent all enumerator values */
 
   /* don't touch this! */
   NUMBER_OF_ERROR_MESSAGES             /* Number of error messages */
