@@ -619,7 +619,7 @@ rlivePoint (eBBlock **ebbs, int count, bool emitWarnings)
                       findNextUse (ebbs[i], ic->next, IC_LEFT(ic));
 
                       // If this is a send extend the LR to the call. For new register allocator we want this for builtin send only.
-                      if (ic->op == SEND && !((TARGET_Z80_LIKE || TARGET_PDK_LIKE || TARGET_IS_STM8) && !ic->builtinSEND))
+                      if (ic->op == SEND && !((TARGET_Z80_LIKE || TARGET_PDK_LIKE || TARGET_IS_STM8 || TARGET_IS_F8) && !ic->builtinSEND))
                         {
                           iCode *lic;
                           for (lic = ic; lic; lic = lic->next)
