@@ -16,23 +16,28 @@
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-   
-   In other words, you are welcome to use, share and improve this program.
-   You are forbidden to forbid anyone else to use, share and improve
-   what you give them.   Help stamp out software-hoarding!  
 -------------------------------------------------------------------------*/
 
 #ifndef SDCCGENHC08_H
 #define SDCCGENHC08_H
 
-enum
+typedef enum
   {
-    AOP_LIT = 1,
-    AOP_REG, AOP_DIR,
-    AOP_STK, AOP_IMMD, AOP_STR,
-    AOP_CRY, 
-    AOP_EXT, AOP_SOF, AOP_DUMMY, AOP_IDX
-  };
+  AOP_INVALID,
+  AOP_LIT = 1,   /* operand is a literal value */
+  AOP_REG,       /* is in registers */
+  AOP_DIR,       /* operand using direct addressing mode */
+  AOP_STK,       /* should be pushed on stack this
+                    can happen only for the result */
+  AOP_IMMD,      /* immediate value for eg. remateriazable */
+  AOP_STR,       /* array of strings */
+  AOP_CRY,       /* carry contains the value of this */
+  AOP_EXT,       /* operand using extended addressing mode */
+  AOP_SOF,       /* operand at an offset on the stack */
+  AOP_DUMMY,     /* Read undefined, discard writes */
+  AOP_IDX        /* operand using indexed addressing mode */
+  }
+AOP_TYPE;
 
 enum
   {
@@ -45,24 +50,10 @@ enum
    in */
 typedef struct asmop
   {
-
-    short type;			
-    /* can have values
-       AOP_LIT    -  operand is a literal value
-       AOP_REG    -  is in registers
-       AOP_DIR    -  operand using direct addressing mode
-       AOP_STK    -  should be pushed on stack this
-       can happen only for the result
-       AOP_IMMD   -  immediate value for eg. remateriazable 
-       AOP_CRY    -  carry contains the value of this
-       AOP_STR    -  array of strings
-       AOP_SOF    -  operand at an offset on the stack
-       AOP_EXT    -  operand using extended addressing mode
-       AOP_IDX    -  operand using indexed addressing mode
-    */
-    short regmask;              /* register mask if AOP_REG */
+    AOP_TYPE type;		
     short coff;			/* current offset */
     short size;			/* total size */
+    short regmask;              /* register mask if AOP_REG */
     operand *op;		/* originating operand */
     unsigned code:1;		/* is in Code space */
     unsigned freed:1;		/* already freed    */
@@ -73,10 +64,7 @@ typedef struct asmop
 	value *aop_lit;		/* if literal */
 	reg_info *aop_reg[4];	/* array of registers */
 	char *aop_dir;		/* if direct  */
-	struct {
-		char *aop_immd1;	/* if immediate others are implied */
-		char *aop_immd2;	/* cast remat will generate this   */
-	} aop_immd;
+        char *aop_immd;         /* if immediate */
 	int aop_stk;		/* stack offset when AOP_STK */
       }
     aopu;
