@@ -269,20 +269,6 @@ packRegsForAssign (iCode *ic, eBBlock *ebp)
         }
     }
 
-  /* if the result is on stack or iaccess then it must be
-     the same as at least one of the operands */
-  if (OP_SYMBOL (IC_RESULT (ic))->onStack || OP_SYMBOL (IC_RESULT (ic))->iaccess)
-    {
-      /* the operation has only one symbol
-         operator then we can pack */
-      if ((IC_LEFT (dic) && !IS_SYMOP (IC_LEFT (dic))) || (IC_RIGHT (dic) && !IS_SYMOP (IC_RIGHT (dic))))
-        goto pack;
-
-      if (!((IC_LEFT (dic) &&
-             IC_RESULT (ic)->key == IC_LEFT (dic)->key) || (IC_RIGHT (dic) && IC_RESULT (ic)->key == IC_RIGHT (dic)->key)))
-        return 0;
-    }
-pack:
   /* found the definition */
 
   /* delete from liverange table also
@@ -344,9 +330,12 @@ packRegsForOneuse (iCode *ic, operand **opp, eBBlock *ebp)
   if (dic->seq < ebp->fSeq || dic->seq > ebp->lSeq)
     return 0;                /* non-local */
 
-  /* for now handle results from assignments from globals only */
-  if (!(dic->op == '=' || dic->op == CAST && SPEC_USIGN (getSpec (operandType (IC_RIGHT (dic)))) && operandSize (op) > operandSize (IC_RIGHT (dic)))
-    || !isOperandGlobal (IC_RIGHT (dic)))
+  /* for now handle results from assignments from globals, and from 8-bit __sfr only */
+  if (!(dic->op == '=' || dic->op == CAST && SPEC_USIGN (getSpec (operandType (IC_RIGHT (dic)))) && operandSize (op) >= operandSize (IC_RIGHT (dic)) && (!IS_BITINT (OP_SYM_TYPE (IC_RIGHT (dic))) || !(SPEC_BITINTWIDTH (OP_SYM_TYPE (IC_RIGHT (dic))) % 8)))
+    || !(isOperandGlobal (IC_RIGHT (dic)) || IS_SYMOP (IC_RIGHT (dic)) && IN_REGSP (SPEC_OCLS (OP_SYMBOL (IC_RIGHT (dic))->etype)) && operandSize (op) == 1 && operandSize (IC_RIGHT (dic)) == 1))
+    return 0;
+
+  if ((IS_OP_VOLATILE (IC_LEFT (ic)) || IS_OP_VOLATILE (IC_RIGHT (ic))) && IS_OP_VOLATILE (IC_RIGHT (dic))) // Avoid swapping read order on volatiles.
     return 0;
 
   if (IS_OP_VOLATILE (IC_RESULT (ic)) && IS_OP_VOLATILE (IC_RIGHT (dic))) // Only case with two volatiles that we can optimize: Some bitwise operation on __sfr.
@@ -391,7 +380,7 @@ packRegsForOneuse (iCode *ic, operand **opp, eBBlock *ebp)
     }
 
   /* Optimize out the assignment */
-  *opp = operandFromOperand (IC_RIGHT(dic));
+  *opp = operandFromOperand (IC_RIGHT (dic));
   (*opp)->isaddr = true;
   
   bitVectUnSetBit (OP_SYMBOL (op)->defs, dic->key);
@@ -502,14 +491,14 @@ packRegisters (eBBlock * ebp)
       /* In some cases redundant moves can be eliminated */
       if (ic->op == GET_VALUE_AT_ADDRESS || ic->op == SET_VALUE_AT_ADDRESS ||
         ic->op == '+' || ic->op == '-' || ic->op == UNARYMINUS ||
-        ic->op == '|' || ic->op == '&' || ic->op == '^' ||
+        ic->op == '|' || ic->op == BITWISEAND || ic->op == '^' ||
         ic->op == EQ_OP || ic->op == NE_OP ||
         ic->op == IFX && operandSize (IC_COND (ic)) == 1 ||
         ic->op == IPUSH && operandSize (IC_LEFT (ic)) == 1 ||
         ic->op == LEFT_OP || ic->op == RIGHT_OP)
         packRegsForOneuse (ic, &(IC_LEFT (ic)), ebp);
       if (ic->op == '+' || ic->op == '-' ||
-        ic->op == '|' || ic->op == '&' || ic->op == '^' ||
+        ic->op == '|' || ic->op == BITWISEAND || ic->op == '^' ||
         ic->op == EQ_OP || ic->op == NE_OP ||
         ic->op == LEFT_OP || ic->op == RIGHT_OP)
         packRegsForOneuse (ic, &(IC_RIGHT (ic)), ebp);
