@@ -101,7 +101,7 @@ static bool operand_byte_in_reg(const operand *o, int offset, reg_t r, const ass
   if(!o || !IS_SYMOP(o))
     return(false);
 
-  operand_map_t::const_iterator oi, oi2, oi3, oi_end;
+  operand_map_t::const_iterator oi, oi_end;
 
   for(boost::tie(oi, oi_end) = G[i].operands.equal_range(OP_SYMBOL_CONST(o)->key); offset && oi != oi_end; offset--, oi++);
 
@@ -140,15 +140,15 @@ static bool Ainst_ok(const assignment &a, unsigned short int i, const G_t &G, co
   bool right_dir = IS_TRUE_SYMOP (right) || IS_ITEMP (right) && !(options.stackAuto || reentrant) && !right_in_A;
 
   // For some iCodes, code generation can handle anything.
-  if (ic->op == GETBYTE || ic->op == '=' || ic->op == DUMMY_READ_VOLATILE || ic->op == CAST || ic->op == GET_VALUE_AT_ADDRESS || ic->op == SET_VALUE_AT_ADDRESS || ic->op == '~' || ic->op == '|' || ic->op == '^' || ic->op == BITWISEAND && !ifxForOp (result, ic))
+  if (ic->op == GETBYTE || ic->op == '=' || ic->op == DUMMY_READ_VOLATILE || ic->op == CAST || ic->op == GET_VALUE_AT_ADDRESS || ic->op == SET_VALUE_AT_ADDRESS || ic->op == '~' || ic->op == '|' || ic->op == '^' || ic->op == BITWISEAND)
     return(true);
 
   if(result && IS_ITEMP(result) && OP_SYMBOL_CONST(result)->remat && !operand_in_reg(result, REG_A, ia, i, G) && !operand_in_reg(result, REG_P, ia, i, G))
     return(true);
 
   if ((ic->op == EQ_OP || ic->op == NE_OP) && dying_A &&
-    (left_in_A && (right_dir || IS_OP_LITERAL(right) || IS_ITEMP(right) && OP_SYMBOL_CONST(right)->remat) ||
-    right_in_A && (left_dir || IS_OP_LITERAL(left) || IS_ITEMP(left) && OP_SYMBOL_CONST(left)->remat)))
+    (left_in_A && (right_dir || IS_OP_LITERAL (right) || IS_ITEMP(right) && OP_SYMBOL_CONST (right)->remat) ||
+    right_in_A && (left_dir || IS_OP_LITERAL (left) || IS_ITEMP(left) && OP_SYMBOL_CONST (left)->remat)))
     return (true);
 
   if ((ic->op == EQ_OP || ic->op == NE_OP || (ic->op == '>' || ic->op == '<') && SPEC_USIGN(getSpec(operandType(left)))) && // Non-destructive comparison.
@@ -191,7 +191,7 @@ static bool Ainst_ok(const assignment &a, unsigned short int i, const G_t &G, co
     return (true);
 
   if ((ic->op == LEFT_OP || ic->op == RIGHT_OP))
-    return(IS_OP_LITERAL(right) || right_in_A && !result_in_A);
+    return (IS_OP_LITERAL(right) || right_in_A && !result_in_A);
 
   if (ic->op == '^' &&
     (operand_byte_in_reg(result, 0, REG_A, a, i, G) && (operand_byte_in_reg(left, 0, REG_A, a, i, G) || operand_byte_in_reg(right, 0, REG_A, a, i, G)) ||
@@ -481,7 +481,6 @@ static void get_best_local_assignment_biased(assignment &a, typename boost::grap
 {
   a = *T[t].assignments.begin();
 
-  std::set<var_t>::const_iterator vi, vi_end;
   varset_t newlocal;
   std::set_union(T[t].alive.begin(), T[t].alive.end(), a.local.begin(), a.local.end(), std::inserter(newlocal, newlocal.end()));
   a.local = newlocal;
@@ -685,6 +684,9 @@ iCode *pdk_ralloc2_cc(ebbIndex *ebbi)
 
   if (reentrant)
     {
+      // Try to reuse parameter locations first.
+      mergeSpiltParms(spilt_conflict_graph);
+
       chaitin_salloc(spilt_conflict_graph);
 
       if(options.dump_graphs)
