@@ -573,7 +573,7 @@ PORT stm8_port =
     {4, 5, 5},                  /* sizeofMatchJump[] - assuming operand in reg, inverse can be optimized away - would be much higher otherwise */
     {4, 5, 5},                  /* sizeofRangeCompare[] - same as above */
     3,                          /* sizeofSubtract - assuming 2 byte index, would be 2 otherwise */
-    5,                          /* sizeofDispatch - 1 byte for sllw followed by 3 bytes for ldw x, (..., X) and 2 byte for jp (x) */
+    5,                          /* sizeofDispatch - 1 byte for sllw followed by 3 bytes for ldw x, (..., x) and 1 byte for jp (x) */
   },
   "_",
   stm8_init,
