@@ -647,7 +647,7 @@ static bool argCont(const char *arg, char what)
 static bool
 stm8MightReadFlag(const lineNode *pl, const char *what)
 {
-  if (strcmp (what, "c") && strcmp (what, "n") && strcmp (what, "z"))
+  if (strcmp (what, "v") && strcmp (what, "c") && strcmp (what, "n") && strcmp (what, "z"))
     return true;
 
   if (ISINST (pl->line, "push"))
@@ -844,7 +844,7 @@ stm8MightRead(const lineNode *pl, const char *what)
   if (ISINST (pl->line, "call") || ISINST (pl->line, "callr") || ISINST (pl->line, "callf"))
     {
       const symbol *f = findSym (SymbolTab, 0, pl->line + (ISINST (pl->line, "call") ? 6 : 7));
-      if (f)
+      if (f && IS_FUNC (f->type))
         return stm8IsParmInCall(f->type, what);
       else // Fallback needed for calls through function pointers and for calls to literal addresses.
         return (stm8MightBeParmInCallFromCurrentFunction(what) || stm8MightBeParmInPCallFromCurrentFunction(what));
@@ -875,6 +875,13 @@ stm8CondJump(const lineNode *pl)
 static bool
 stm8SurelyWritesFlag(const lineNode *pl, const char *what)
 {
+  // according to calling convention caller has to save flags
+  if(ISINST(pl->line, "ret") || ISINST(pl->line, "retf") ||
+     ISINST(pl->line, "call") || ISINST(pl->line, "callf") ||
+     ISINST(pl->line, "jp") && findSym (SymbolTab, 0, pl->line + 4) ||
+     ISINST(pl->line, "jpf") && findSym (SymbolTab, 0, pl->line + 5))
+    return true;
+
   if (!strcmp (what, "v") || !strcmp (what, "c"))
     {        
       if (ISINST (pl->line, "addw") && !strcmp (pl->line + 5, "sp"))
@@ -943,6 +950,36 @@ stm8SurelyWritesFlag(const lineNode *pl, const char *what)
   return false;
 }
 
+static bool
+callSurelyWrites (const lineNode *pl, const char *what)
+{
+  const symbol *f = 0;
+  if ((ISINST(pl->line, "call") || ISINST(pl->line, "callf")) && !strchr(pl->line, ','))
+    f = findSym (SymbolTab, 0, pl->line + 6 + ISINST(pl->line, "callf"));
+  else if ((ISINST(pl->line, "jp") || ISINST(pl->line, "jr") || ISINST(pl->line, "jpf")) && !strchr(pl->line, ','))
+    f = findSym (SymbolTab, 0, pl->line + 4 + ISINST(pl->line, "jpf"));
+
+  const bool *preserved_regs;
+
+  if(f)
+    preserved_regs = f->type->funcAttrs.preserved_regs;
+  else // Err on the safe side for jp and jr - might not be a function call, might e.g. be a jump table.
+    return (false);
+
+  if (!strcmp (what, "a"))
+    return !preserved_regs[A_IDX];
+  else if (!strcmp (what, "xl"))
+    return !preserved_regs[XL_IDX];
+  else if (!strcmp (what, "xh"))
+    return !preserved_regs[XH_IDX];
+  else if (!strcmp (what, "yl"))
+    return !preserved_regs[YL_IDX];
+  else if (!strcmp (what, "yh"))
+    return !preserved_regs[YH_IDX];
+
+  return false;
+}
+
 static bool
 stm8SurelyWrites(const lineNode *pl, const char *what)
 {
@@ -1035,6 +1072,9 @@ stm8SurelyWrites(const lineNode *pl, const char *what)
         return true;
     }
 
+  if (ISINST(pl->line, "call"))
+    return (callSurelyWrites (pl, what));
+
   return false;
 }
 
@@ -1120,12 +1160,27 @@ scan4op (lineNode **pl, const char *what, const char *untilOp,
 
       if(stm8UncondJump(*pl))
         {
-          *pl = findLabel (*pl);
-            if (!*pl)
-              {
-                D(("S4O_ABORT at unconditional jump\n"));
-                return S4O_ABORT;
-              }
+          lineNode *tlbl = findLabel (*pl);
+          if (!tlbl) // jp/jr could be a tail call.
+            {
+              const symbol *f = findSym (SymbolTab, 0, (*pl)->line + 4 + ISINST ((*pl)->line, "jpf"));
+              if (f && stm8IsParmInCall(f->type, what))
+                {
+                  D (("S4O_RD_OP\n"));
+                  return S4O_RD_OP;
+                }
+              else if(callSurelyWrites (*pl, what))
+                {
+                  D (("S4O_WR_OP\n"));
+                  return S4O_WR_OP;
+                }
+            }
+          *pl = tlbl;
+          if (!*pl)
+            {
+              D(("S4O_ABORT at unconditional jump\n"));
+              return S4O_ABORT;
+            }
         }
       if(stm8CondJump(*pl))
         {
