@@ -251,8 +251,7 @@ z80MightReadFlag(const lineNode *pl, const char *what)
      ISINST(pl->line, "rld") ||
      ISINST(pl->line, "rrd") ||
      ISINST(pl->line, "mlt") ||
-     ISINST(pl->line, "multu") ||
-     ISINST(pl->line, "multuw"))
+     ISINST(pl->line, "out"))
     return false;
   if(ISINST(pl->line, "halt") ||
      ISINST(pl->line, "rlca") ||
@@ -265,18 +264,27 @@ z80MightReadFlag(const lineNode *pl, const char *what)
      ISINST(pl->line, "ldir") ||
      ISINST(pl->line, "outd") ||
      ISINST(pl->line, "outi") ||
-     ISINST(pl->line, "jdnz"))
+     ISINST(pl->line, "djnz"))
+    return false;
+
+  if(IS_RAB &&
+    ISINST(pl->line, "bool"))
     return false;
 
   if(IS_SM83 &&
-     (ISINST(pl->line, "stop") ||
-      ISINST(pl->line, "ldh")))
+    (ISINST(pl->line, "stop") ||
+    ISINST(pl->line, "ldh")))
     return false;
 
   if((IS_SM83 || IS_Z80N) &&
       ISINST(pl->line, "swap"))
     return false;
 
+  if(IS_R800 &&
+     (ISINST(pl->line, "multu") ||
+     ISINST(pl->line, "multuw")))
+    return false;
+
   if(ISINST(pl->line, "rl") ||
      ISINST(pl->line, "rr") ||
      ISINST(pl->line, "rla") ||
@@ -319,7 +327,7 @@ z80MightReadFlag(const lineNode *pl, const char *what)
   if (IS_SM83 && ISINST(pl->line, "ldhl"))
     return false;
     
-  if(IS_RAB && (ISINST(pl->line, "ioi") || ISINST(pl->line, "ioe")))
+  if(IS_RAB && (ISINST(pl->line, "ioi") || ISINST(pl->line, "ioe") || ISINST(pl->line, "ipres")))
     return(false);
 
   if(IS_Z80N &&
@@ -330,7 +338,11 @@ z80MightReadFlag(const lineNode *pl, const char *what)
     ISINST(pl->line, "brlc")))
     return(false);
 
-  return true;
+  if((IS_Z180 || IS_EZ80_Z80 || IS_Z80N) &&
+    ISINST(pl->line, "tst"))
+    return(false);
+ 
+  return true; // Fail-safe: we have no idea what happens at this line, so assume it might read anything.
 }
 
 static bool
@@ -360,7 +372,7 @@ z80MightRead(const lineNode *pl, const char *what)
   if(ISINST(pl->line, "call") && strchr(pl->line, ',') == 0)
     {
       const symbol *f = findSym (SymbolTab, 0, pl->line + 6);
-      if (f)
+      if (f && IS_FUNC (f->type))
         return z80IsParmInCall(f->type, what);
       else // Fallback needed for calls through function pointers and for calls to literal addresses.
         return z80MightBeParmInCallFromCurrentFunction(what);
@@ -384,7 +396,9 @@ z80MightRead(const lineNode *pl, const char *what)
     {
       if(argCont(strchr(pl->line, ','), what))
         return(true);
-      if(*(strchr(pl->line, ',') - 1) == ')' && strstr(pl->line + 3, what) && (strchr(pl->line, '#') == 0 || strchr(pl->line, '#') > strchr(pl->line, ',')))
+      if(*(strchr(pl->line, ',') - 1) == ')' && strstr(pl->line + 3, what) &&
+        (strchr(pl->line, '#') == 0 || strchr(pl->line, '#') > strchr(pl->line, ',')) &&
+        (strchr(pl->line, '_') == 0 || strchr(pl->line, '_') > strchr(pl->line, ',')))
         return(true);
       if (!strcmp(what, "sp") && strchr(pl->line, '(')) // Assume any indirect memory access to be a stack access. This avoids optimizing out stackframe setups for local variables (bug #3173).
         return(true);
@@ -720,22 +734,34 @@ z80SurelyWritesFlag(const lineNode *pl, const char *what)
   if(ISINST(pl->line, "daa"))
     return (!!strcmp(what, "nf"));
 
-  if(ISINST(pl->line, "scf") ||
-     ISINST(pl->line, "ccf"))
-    return (!!strcmp(what, "zf") && !!strcmp(what, "sf") && !!strcmp(what, "pf"));
+  if(ISINST(pl->line, "ccf") ||
+    ISINST(pl->line, "scf"))
+    return (!strcmp(what, "hf") || !strcmp(what, "nf") || !strcmp(what, "cf"));
 
   if(ISINST(pl->line, "cpl"))
-    return (!!strcmp(what, "zf") && !!strcmp(what, "cf"));
+    return (!strcmp(what, "hf") || !strcmp(what, "nf"));
 
-  // only for simple registers
-  if((ISINST(pl->line, "inc") ||
-      ISINST(pl->line, "dec")) && (strlen(pl->line+4) == 1))
-    return (!!strcmp(what, "cf"));
+  if(ISINST(pl->line, "inc") || ISINST(pl->line, "dec"))
+    {
+      // 8-bit inc affects all flags other than c.
+      if (strlen(pl->line + 4) == 1 || // 8-bit register
+        !strcmp(pl->line + 4, "(hl)") ||
+        !strcmp(pl->line + 6, "(ix)") ||
+        !strcmp(pl->line + 6, "(iy)"))
+        return (!!strcmp(what, "cf"));
+      return false; // 16-bit inc does not affect flags.
+    }
 
   if(ISINST(pl->line, "add"))
     return (argCont(pl->line + 4, "a") ||
            (!!strcmp(what, "zf") && !!strcmp(what, "sf") && !!strcmp(what, "pf")));
 
+  if(ISINST(pl->line, "ldd") ||
+    ISINST(pl->line, "lddr") ||
+    ISINST(pl->line, "ldi") ||
+    ISINST(pl->line, "ldir"))
+    return (!strcmp(what, "hf") || !strcmp(what, "pf") || !strcmp(what, "nf"));
+
   // pop af writes
   if(ISINST(pl->line, "pop"))
     return (argCont(pl->line + 4, "af"));
@@ -745,22 +771,102 @@ z80SurelyWritesFlag(const lineNode *pl, const char *what)
      ISINST(pl->line, "call"))
     return true;
 
+  if(ISINST(pl->line, "rld") ||
+    ISINST(pl->line, "rrd"))
+    return (!strcmp(what, "hf") || !strcmp(what, "pf") || !strcmp(what, "nf"));
+
+  if(ISINST(pl->line, "djnz") ||
+    ISINST(pl->line, "ex") ||
+    ISINST(pl->line, "nop") ||
+    ISINST(pl->line, "out") ||
+    ISINST(pl->line, "push") ||
+    ISINST(pl->line, "res") ||
+    ISINST(pl->line, "set"))
+    return false;
+
   if(IS_SM83 &&
      (ISINST(pl->line, "swap") ||
       ISINST(pl->line, "ldhl") ||
       ISINST(pl->line, "lda")))
     return true;
 
+  if(IS_Z80N &&
+    ISINST(pl->line, "swap"))
+    return false;
+
   /* handle IN0 r,(n) and IN r,(c) instructions */
   if(ISINST(pl->line, "in0") || (!strncmp(pl->line, "in\t", 3) &&
      (!strcmp(pl->line+5, "(c)") || !strcmp(pl->line+5, "(bc)"))))
     return (!!strcmp(what, "cf"));
 
-  return false;
+  if(IS_RAB &&
+    ISINST(pl->line, "bool"))
+    return true;
+
+  if(IS_RAB &&
+    ISINST(pl->line, "ipres"))
+    return false;
+
+  if(ISINST(pl->line, "mlt"))
+    return (!IS_Z80N);
+
+  if((IS_Z180 || IS_EZ80_Z80 || IS_Z80N) &&
+    ISINST(pl->line, "tst"))
+    return true;
+
+  return false; // Fail-safe: we have no idea what happens at this line, so assume it writes nothing.
 }
 
 static bool
-z80SurelyWrites(const lineNode *pl, const char *what)
+callSurelyWrites (const lineNode *pl, const char *what)
+{
+  const symbol *f = 0;
+  if (ISINST(pl->line, "call") && !strchr(pl->line, ','))
+    f = findSym (SymbolTab, 0, pl->line + 6);
+  else if ((ISINST(pl->line, "jp") || ISINST(pl->line, "jr")) && !strchr(pl->line, ','))
+    f = findSym (SymbolTab, 0, pl->line + 4);
+
+  const bool *preserved_regs;
+
+  if (f && (strlen(what) == 2 && what[1] == 'f')) // Flags are never preserved across function calls.
+    return(true);
+
+  if(!strcmp(what, "ix"))
+    return(false);
+
+  if(f)
+    preserved_regs = f->type->funcAttrs.preserved_regs;
+  else if (ISINST(pl->line, "call"))
+    preserved_regs = z80_regs_preserved_in_calls_from_current_function;
+  else // Err on the safe side for jp and jr - might not be a function call, might e.g. be a jump table.
+    return (false);
+
+  if (!strcmp (what, "a"))
+    return !preserved_regs[A_IDX];
+  if (!strcmp (what, "c"))
+    return !preserved_regs[C_IDX];
+  if (!strcmp (what, "b"))
+    return !preserved_regs[B_IDX];
+  if (!strcmp (what, "e"))
+    return !preserved_regs[E_IDX];
+  if (!strcmp (what, "d"))
+    return !preserved_regs[D_IDX];
+  if (!strcmp (what, "l"))
+    return !preserved_regs[L_IDX];
+  if (!strcmp (what, "h"))
+    return !preserved_regs[H_IDX];
+  if (!strcmp (what, "iyl"))
+    return !preserved_regs[IYL_IDX];
+  if (!strcmp (what, "iyh"))
+    return !preserved_regs[IYH_IDX];
+  if (!strcmp (what, "iy"))
+    return !preserved_regs[IYL_IDX] && !preserved_regs[IYH_IDX];
+
+  return (false);
+}
+
+static bool
+z80SurelyWrites (const lineNode *pl, const char *what)
 {
   if(strcmp(what, "iyl") == 0 || strcmp(what, "iyh") == 0)
     what = "iy";
@@ -795,34 +901,9 @@ z80SurelyWrites(const lineNode *pl, const char *what)
 
   if(ISINST(pl->line, "pop") && strstr(pl->line + 4, what))
     return(true);
-  if(ISINST(pl->line, "call") && strchr(pl->line, ',') == 0)
-    {
-      const symbol *f = findSym (SymbolTab, 0, pl->line + 6);
-      const bool *preserved_regs;
+  if (ISINST(pl->line, "call") && strchr(pl->line, ',') == 0)
+    return (callSurelyWrites (pl, what));
 
-      if(!strcmp(what, "ix"))
-        return(false);
-
-      if(f)
-          preserved_regs = f->type->funcAttrs.preserved_regs;
-      else // Err on the safe side.
-        preserved_regs = z80_regs_preserved_in_calls_from_current_function;
-
-      if(!strcmp(what, "c"))
-        return !preserved_regs[C_IDX];
-      if(!strcmp(what, "b"))
-        return !preserved_regs[B_IDX];
-      if(!strcmp(what, "e"))
-        return !preserved_regs[E_IDX];
-      if(!strcmp(what, "d"))
-        return !preserved_regs[D_IDX];
-      if(!strcmp(what, "l"))
-        return !preserved_regs[L_IDX];
-      if(!strcmp(what, "h"))
-        return !preserved_regs[H_IDX];
-      if(!strcmp(what, "iy"))
-        return !preserved_regs[IYL_IDX] && !preserved_regs[IYH_IDX];
-    }
   if(strcmp(pl->line, "ret") == 0)
     return true;
 
@@ -934,49 +1015,64 @@ scan4op (lineNode **pl, const char *what, const char *untilOp,
 
       if(isFlag)
         {
-        if(z80MightReadFlag(*pl, what))
-          {
-            D(("S4O_RD_OP (flag)\n"));
-            return S4O_RD_OP;
-          }
+          if(z80MightReadFlag(*pl, what))
+            {
+              D(("S4O_RD_OP (flag)\n"));
+              return S4O_RD_OP;
+            }
         }
       else
         {
-        if(z80MightRead(*pl, what))
-          {
-            D(("S4O_RD_OP\n"));
-            return S4O_RD_OP;
-          }
+          if (z80MightRead (*pl, what))
+            {
+              D (("S4O_RD_OP\n"));
+              return S4O_RD_OP;
+            }
         }
 
-      if(z80UncondJump(*pl))
+      if (z80UncondJump (*pl))
         {
-          *pl = findLabel (*pl);
-            if (!*pl)
-              {
-                D(("S4O_ABORT\n"));
-                return S4O_ABORT;
-              }
+          lineNode *tlbl = findLabel (*pl);
+          if (!tlbl) // jp/jr could be a tail call.
+            {
+              const symbol *f = findSym (SymbolTab, 0, (*pl)->line + 4);
+              if (f && z80IsParmInCall(f->type, what))
+                {
+                  D (("S4O_RD_OP\n"));
+                  return S4O_RD_OP;
+                }
+              else if(callSurelyWrites (*pl, what))
+                {
+                  D (("S4O_WR_OP\n"));
+                  return S4O_WR_OP;
+                }
+            }
+          *pl = tlbl;
+          if (!*pl)
+            {
+              D (("S4O_ABORT\n"));
+              return S4O_ABORT;
+            }
         }
-      if(z80CondJump(*pl))
+      if (z80CondJump(*pl))
         {
           *plCond = findLabel (*pl);
           if (!*plCond)
             {
-              D(("S4O_ABORT\n"));
+              D (("S4O_ABORT\n"));
               return S4O_ABORT;
             }
-          D(("S4O_CONDJMP\n"));
+          D (("S4O_CONDJMP\n"));
           return S4O_CONDJMP;
         }
 
-      if(isFlag)
+      if (isFlag)
         {
-        if(z80SurelyWritesFlag(*pl, what))
-          {
-            D(("S4O_WR_OP (flag)\n"));
-            return S4O_WR_OP;
-          }
+          if (z80SurelyWritesFlag (*pl, what))
+            {
+              D (("S4O_WR_OP (flag)\n"));
+              return S4O_WR_OP;
+            }
         }
       else
         {
@@ -1653,6 +1749,9 @@ int z80instructionSize(lineNode *pl)
     ISINST(pl->line, "brlc")))
     return(2);
 
+  if(IS_Z80N && ISINST(pl->line, "nextreg"))
+    return(op2start && !STRNCASECMP(op2start, "a", 1) ? 3 : 4);
+
   if(ISINST(pl->line, ".db") || ISINST(pl->line, ".byte"))
     {
       int i, j;
