@@ -12,6 +12,8 @@
 #define ADDR 0x0100
 #elif defined(__SDCC_stm8)
 #define ADDR 0x1000
+#elif defined(__SDCC_f8)
+#define ADDR 0x3800
 #elif defined(__SDCC_pdk14) || defined(__SDCC_pdk15)
 #define ADDR 0x70
 #else
