@@ -1,5 +1,5 @@
 /* bug-2708.c
-   The STM8 peephole optimizer didn't like indirect adressing modes in inline asm.
+   The STM8 peephole optimizer didn't like indirect addressing modes in inline asm.
  */
 
 #include <testfwk.h>
@@ -65,7 +65,7 @@ void testBug(void)
 {
 	uint8_t c;
 	cp = &c;
-#ifndef __SDCC
+
 	c = 0xaa;
 	f1();
 	ASSERT(c == 0x55);
@@ -85,6 +85,5 @@ void testBug(void)
 	c = 0xaa;
 	f5();
 	ASSERT(c == 0x55);
-#endif
 }
 
