@@ -1,11 +1,13 @@
 /* bug-3081.c
-   z80 backend inserted extra whitespace in call vie __z88dk_fastcall function pointer 
+   z80 backend inserted extra whitespace in call via __z88dk_fastcall function pointer 
    for --reserve-regs-iy, confusing peephole optimizer
  */
 
 
 #include <testfwk.h>
 
+#pragma disable_warning 283
+
 #include <stdint.h>
 
 #if defined(__SDCC_mcs51) || defined(__SDCC_pdk14) || defined(__SDCC_pdk15)
