@@ -10,6 +10,10 @@
 # define ADDRESS(x) (0x01A0 + 0x ## x)
 #elif defined(__SDCC_stm8)
 # define ADDRESS(x) (0x10 ## x)
+#elif defined(__SDCC_f8)
+# define ADDRESS(x) (0x30 ## x)
+#elif defined( __SDCC_pdk14) || defined( __SDCC_pdk15) // No RAM above 0x7f / 0xff.
+# define ADDRESS(x) (0x78 + 0x ## x)
 #else
 # define ADDRESS(x) (0xCA ## x)
 #endif
@@ -17,6 +21,10 @@
 /* This produces incorrect code */
 #define REG_1 (*(__xdata unsigned char*)ADDRESS(00))
 
+// Absolute addresses should use an initial value to make sure compiler does not use these addresses.
+// From manual: If you provide an initializer actual memory allocation will take place and overlaps will be detected by the linker
+__xdata unsigned char  __at(ADDRESS(00)) REG_1_RES = 0;
+
 void incorrect(void)
 {
 	REG_1 |= 1;
@@ -24,7 +32,7 @@ void incorrect(void)
 }
 
 /* This produces correct code */
-__xdata unsigned char __at(ADDRESS(01)) REG_2;
+__xdata unsigned char __at(ADDRESS(01)) REG_2 = 0;
 
 void correct(void)
 {
@@ -34,7 +42,6 @@ void correct(void)
 
 void testBug(void)
 {
-#if !defined( __SDCC_pdk14) && !defined( __SDCC_pdk15) // No RAM above 0x7f / 0xff.
 #ifdef __SDCC
 	REG_1 = 0x40;
 	incorrect();
@@ -43,5 +50,4 @@ void testBug(void)
 	REG_2 = 0x50;
 	correct();
 	ASSERT (REG_2 == 0x53);
-#endif
 }
