@@ -8,7 +8,7 @@ cvt-1.c from the execute part of the gcc torture tests.
 #pragma disable_warning 93
 #endif
 
-#if !defined(__SDCC_pdk14) && !defined(__SDCC_pdk15) // Bug #2874
+#if !defined(__SDCC_pdk14) && !defined(__SDCC_pdk15) // Lack of memory
 static inline long
 g1 (double x)
 {
