@@ -8,41 +8,49 @@
 #pragma std_c99
 #endif
 
+int
 div1 (signed char x)
 {
   return x / -1;
 }
 
+int
 div2 (signed short x)
 {
   return x / -1;
 }
 
+int
 div3 (signed char x, signed char y)
 {
   return x / y;
 }
 
+int
 div4 (signed short x, signed short y)
 {
   return x / y;
 }
 
+int
 mod1 (signed char x)
 {
   return x % -1;
 }
 
+int
 mod2 (signed short x)
 {
   return x % -1;
 }
 
+int
 mod3 (signed char x, signed char y)
 {
   return x % y;
 }
 
+int
 mod4 (signed short x, signed short y)
 {
   return x % y;
@@ -55,7 +63,7 @@ mod5 (signed long x, signed long y)
 {
   return x % y;
 }
-     
+
 unsigned long
 mod6 (unsigned long x, unsigned long y)
 {
@@ -63,34 +71,24 @@ mod6 (unsigned long x, unsigned long y)
 }
 #endif
 #endif
-     
+
 void
 testTortureExecute (void)
 {
-  if (div1 (-(1 << 7)) != 1 << 7)
-    ASSERT (0);
-  if (div2 (-(1 << 15)) != 1 << 15)
-    ASSERT (0);
-  if (div3 (-(1 << 7), -1) != 1 << 7)
-    ASSERT (0);
-  if (div4 (-(1 << 15), -1) != 1 << 15)
-    ASSERT (0);
-  if (mod1 (-(1 << 7)) != 0)
-    ASSERT (0);
-  if (mod2 (-(1 << 15)) != 0)
-    ASSERT (0);
-  if (mod3 (-(1 << 7), -1) != 0)
-    ASSERT (0);
-  if (mod4 (-(1 << 15), -1) != 0)
-    ASSERT (0);
+  ASSERT(!(div1 (-(1 << 7)) != 1 << 7));
+  ASSERT(!(div2 (-(1 << 15)) != 1 << 15));
+  ASSERT(!(div3 (-(1 << 7), -1) != 1 << 7));
+  ASSERT(!(div4 (-(1 << 15), -1) != 1 << 15));
+  ASSERT(!(mod1 (-(1 << 7)) != 0));
+  ASSERT(!(mod2 (-(1 << 15)) != 0));
+  ASSERT(!(mod3 (-(1 << 7), -1) != 0));
+  ASSERT(!(mod4 (-(1 << 15), -1) != 0));
 #ifndef __SDCC_pdk14 // Lack of memory
 #if !(defined (__SDCC_pdk15) && defined(__SDCC_STACK_AUTO)) // Lack of code memory
-  if (mod5 (0x50000000, 2) != 0)
-    ASSERT (0);
-  if (mod6 (0x50000000, 2) != 0)
-    ASSERT (0);
+  ASSERT(!(mod5 (0x50000000, 2) != 0));
+  ASSERT(!(mod6 (0x50000000, 2) != 0));
 #endif
 #endif
-  
+
   return;
 }
