@@ -6,12 +6,14 @@
 
 #ifdef __SDCC
 #pragma std_c99
+#pragma disable_warning 93
 #endif
 
 /* PR target/64979 */
 
 #include <stdarg.h>
 
+#ifndef __SDCC_pdk14 // Lack of memory
 void
 bar (int x, va_list *ap)
 {
@@ -36,10 +38,13 @@ foo (int x, ...)
   bar (x, (va_list *) ((n == 0) ? ((void *) 0) : &ap));
   va_end (ap);
 }
+#endif
 
 void
 testTortureExecute (void)
 {
+#ifndef __SDCC_pdk14 // Lack of memory
   foo (100, 1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0.5);
   return;
+#endif
 }
