@@ -8,37 +8,30 @@ strct-varg-1.c from the execute part of the gcc torture tests.
 
 struct s { int x, y; };
 
-#if 0 // TODO: enable when SDCC supports passing struct arguments!
+void
 f (int attr, ...)
 {
   struct s va_values;
   va_list va;
-  int i;
 
   va_start (va, attr);
 
-  if (attr != 2)
-    ASSERT (0);
+  ASSERT(attr == 2);
 
   va_values = va_arg (va, struct s);
-  if (va_values.x != 0xaaaa || va_values.y != 0x5555)
-    ASSERT (0);
+  ASSERT(!(va_values.x != 0xaaaa || va_values.y != 0x5555));
 
   attr = va_arg (va, int);
-  if (attr != 3)
-    ASSERT (0);
+  ASSERT(attr == 3);
 
   va_values = va_arg (va, struct s);
-  if (va_values.x != 0xffff || va_values.y != 0x1111)
-    ASSERT (0);
+  ASSERT(!(va_values.x != 0xffff || va_values.y != 0x1111));
   va_end (va);
 }
-#endif
 
 void
 testTortureExecute (void)
 {
-#if 0 // TODO: enable when SDCC supports passing struct arguments!
   struct s a, b;
 
   a.x = 0xaaaa;
@@ -47,6 +40,5 @@ testTortureExecute (void)
   b.y = 0x1111;
 
   f (2, a, 3, b);
-#endif
   return;
 }
